<?php

/**
 * Copyright (c) 2016, BitKassa B.V.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * @category    BitKassa
 * @package     BitKassa_Bitcoin
 * @author      BitKassa (info@bitkassa.nl) / Mollie B.V. (info@mollie.nl)
 * @version     v1.0 (based on Mollie_Ideal v4.7)
 * @copyright   Copyright (c) 2012 Mollie B.V. (http://www.mollie.nl)
 * @license     http://www.opensource.org/licenses/bsd-license.php  Berkeley Software Distribution License (BSD-License 2)
 *
 **/

class Bitcoin_Payment {

	const MIN_TRANS_AMOUNT = 150;

	protected $merchant_key = null;
	protected $status;
	protected $amount = 0;
	protected $description = null;
	protected $return_url = null;
	protected $report_url = null;
	protected $bitkassa_url = null;
	protected $payment_url = null;
	protected $transaction_id = null;
	protected $paid_status = false;
	protected $consumer_info = array();
	protected $error_message = '';
	protected $error_code = 0;
	protected $api_host = 'https://bitinvoice.eu'; // we're using a proxy for the bitkassa.nl bitmerchant API to support older SSL connections
	protected $api_port = 443;

	public function __construct($merchant_key, $api_host = 'https://bitinvoice.eu', $api_port = 443) {
		$this->setMerchantKey($merchant_key);
		$this->api_host = $api_host;
		$this->api_port = $api_port;
	}

	// Zet een betaling klaar bij BitKassa met report en return urls
	public function createPayment($amount, $description, $return_url, $report_url) {

		if (!$this->setDescription($description)) {
			$this->error_message = "De opgegeven omschrijving \"$description\" is incompleet";
			return false;
		}

		if (!$this->setAmount($amount)) {
			$this->error_message = "Het opgegeven bedrag \"$amount\" is onjuist of te laag";
			return false;
		}

		if (!$this->setReturnURL($return_url)) {
			$this->error_message = "De opgegeven return URL \"$return_url\" is onjuist";
			return false;
		}

		if (!$this->setReportURL($report_url)) {
			$this->error_message = "De opgegeven report URL \"$report_url\" is onjuist";
			return false;
		}

		$query_variables = array(
			'merchant_key' => $this->getMerchantKey(),
			'euro' => $this->getAmount(),
			'description' => $this->getDescription(),
			'report_url' => $this->getReportURL(),
			'return_url' => $this->getReturnURL(),
		);

		$create_json = $this->_sendRequest(
			$this->api_host, $this->api_port, '/bitkassa/json/btcpay/', http_build_query($query_variables, '', '&')
		);

		if (empty($create_json))
			return false;

		$create_object = json_decode($create_json);

		if (!$create_object || !isset($create_object->result) || $create_object->result != 'success') {
			if (isset($create_object->errors) && is_array($create_object->errors))
				$this->error_message = implode(", ", $create_object->errors);
			else
				$this->error_message = "Malformed response from BitKassa-API at {$this->api_host}";
			return false;
		}

		$this->transaction_id = (string) $create_object->transaction_id;
		$this->bitkassa_url = (string) $create_object->url;

		return true;
	}

	// Kijk of er daadwerkelijk betaald is
	public function checkPayment($transaction_id) {
		if (!$this->setTransactionId($transaction_id)) {
			$this->error_message = "Er is een onjuiste transactie-ID opgegeven";
			return false;
		}

		$query_variables = array(
			'merchant_key' => $this->merchant_key,
			'transaction_id' => $this->getTransactionId(),
		);

		$check_json = $this->_sendRequest(
			$this->api_host, $this->api_port, '/bitkassa/json/btcpay/check/', http_build_query($query_variables, '', '&')
		);

		if (empty($check_json))
			return false;

		$check_object = json_decode($check_json);

		if (!$check_object || $check_object->result != 'success') {
			if (!empty($check_object->error))
				$this->error_message = $check_object->error;
			return false;
		}

		$this->status = (string) $check_object->status;
		if ($this->status == 'canceled') // omdat spelling
			$this->status == 'cancelled';
		$this->paid_status = (bool) ($this->status == 'paid' || $this->status == 'confirmed');
		$this->amount = (int) $check_object->amount;
		//$this->consumer_info = (isset($check_object->order->consumer)) ? (array) $check_object->order->consumer : array();

		return true;
	}

	/*
	  PROTECTED FUNCTIONS
	 */

	protected function _sendRequest($host, $port, $path, $data) {
		if (function_exists('curl_init')) {
			return $this->_sendRequestCurl($host, $port, $path, $data);
		} else {
			return $this->_sendRequestFsock($host, $port, $path, $data);
		}
	}

	protected function _sendRequestFsock($host, $port, $path, $data) {
		$hostname = str_replace('ssl://', '', $host);
		$hostname = str_replace('https://', '', $host);
		$fp = @fsockopen($host, $port, $errno, $errstr);
		$buf = '';

		if (!$fp) {
			$this->error_message = 'Kon geen verbinding maken met server: ' . $errstr;
			$this->error_code = 0;

			return false;
		}

		@fputs($fp, "POST $path HTTP/1.0\n");
		@fputs($fp, "Host: $hostname\n");
		@fputs($fp, "Content-type: application/x-www-form-urlencoded\n");
		@fputs($fp, "Content-length: " . strlen($data) . "\n");
		@fputs($fp, "Connection: close\n\n");
		@fputs($fp, $data);

		while (!feof($fp)) {
			$buf .= fgets($fp, 128);
		}

		fclose($fp);

		if (empty($buf)) {
			$this->error_message = 'Zero-sized reply';
			return false;
		} else {
			list($headers, $body) = preg_split("/(\r?\n){2}/", $buf, 2);
		}

		return $body;
	}

	protected function _sendRequestCurl($host, $port, $path, $data) {
		$host = str_replace('ssl://', 'https://', $host);

		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $host . $path);
		curl_setopt($ch, CURLOPT_PORT, $port);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_HEADER, FALSE);
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		curl_setopt($ch, CURLOPT_ENCODING, ""); // Tell server which Encodings (gzip, deflate) we support.
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE); // Suppress certificate errors while using the proxy URL
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE); // (note that the remote SSL certificats are valid, but we'll suppress errors if local cacert.pem file missing)

		$body = curl_exec($ch);

		if (curl_errno($ch) == CURLE_SSL_CACERT) {
			/*
			 * On some servers, the list of installed certificates is outdated or not present at all (the ca-bundle.crt
			 * is not installed). So we tell cURL which certificates we trust. Then we retry the requests.
			 */
			curl_setopt($ch, CURLOPT_CAINFO, dirname(__FILE__) . DIRECTORY_SEPARATOR . "cacert.pem");
			$body = curl_exec($ch);
		}

		if (strpos(curl_error($ch), "certificate subject name '$host' does not match target host") !== FALSE) {
			/*
			 * On some servers, the wildcard SSL certificate is not processed correctly. This happens with OpenSSL 0.9.7
			 * from 2003.
			 */
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			$body = curl_exec($ch);
		}

		curl_close($ch);

		return $body;
	}

	/* Getters & setters */

	public function setMerchantKey($merchant_key) {
		if (is_null($merchant_key) || strlen($merchant_key) == 0) {
			return false;
		}
		return ($this->merchant_key = $merchant_key);
	}

	public function getMerchantKey() {
		return $this->merchant_key;
	}

	public function setAmount($amount) {
		if (!preg_match('~^[0-9]+$~', $amount)) {
			return false;
		}

		if (self::MIN_TRANS_AMOUNT > $amount) {
			return false;
		}

		return ($this->amount = $amount);
	}

	public function getAmount() {
		return $this->amount;
	}

	public function setDescription($description) {
		$description = substr($description, 0, 50);
		return ($this->description = $description);
	}

	public function getDescription() {
		return $this->description;
	}

	public function setReturnURL($return_url) {
		if (!preg_match('|(\w+)://([^/:]+)(:\d+)?(.*)|', $return_url))
			return false;
		return ($this->return_url = $return_url);
	}

	public function getReturnURL() {
		return $this->return_url;
	}

	public function setReportURL($report_url) {
		if (!preg_match('|(\w+)://([^/:]+)(:\d+)?(.*)|', $report_url)) {
			return false;
		}
		return ($this->report_url = $report_url);
	}

	public function getReportURL() {
		return $this->report_url;
	}

	public function setTransactionId($transaction_id) {
		if (empty($transaction_id))
			return false;
		return ($this->transaction_id = $transaction_id);
	}

	public function getTransactionId() {
		return $this->transaction_id;
	}

	public function getBitKassaURL() {
		return $this->bitkassa_url;
	}

	public function getPaymentURL() {
		return (string) $this->payment_url;
	}

	public function getPaidStatus() {
		return $this->paid_status;
	}

	public function getStatus() {
		return $this->status;
	}

	public function getConsumerInfo() {
		return $this->consumer_info;
	}

	public function getErrorMessage() {
		return $this->error_message;
	}

	public function getErrorCode() {
		return $this->error_code;
	}

}
